#!/usr/bin/perl

use strict;
use warnings;
use Archive::Zip;

require "fontpatch.pl";

sub determineSourceZip {
	my @sourcefont = reverse(sort(glob("migu-1m-????????.zip")));
	print "$sourcefont[0]\n";
	$sourcefont[0] or die "Cannot find Migu 1M distribution ZIP archive. Please put migu-1m-????????.zip here first.\n";
}

sub extractSourceZip {
	my $zip = Archive::Zip->new($_[0]);
	my @members = $zip->memberNames();
	foreach (@members) {
		print "Extract: $_\n";
		$zip->extractMember($_, "./$_");
	}
	$_[0] =~ s/\..*$//;
	$_[0];
}

sub ttf2sfd {
	print "Convert font $_[1]...\n";
	my $result = system "fontforge -lang=ff -c \"Open('$_[0]/$_[1].ttf');Save('$_[1].sfd')\"";
	die "Something went wrong...\n" if ($result);
}

sub patchGlyph {
	my $weight = $_[0];
	print "Patching $weight...\n";
	my %patchDat = (regular => \%PatchData::regSpline, bold => \%PatchData::boldSpline);

	open IN, "migu-1m-$weight.sfd" or die "Cannot open migu-1m-$weight.sfd\n";
	open OUT, ">hemiplatia-$weight.sfd" or die "Cannot write to hemiplatia-$weight.sfd\n";
	my $skipFlag = 0;
	my $currentGlyph = '';
	while (<IN>) {
		study;
		($currentGlyph = $_) =~ s/^StartChar: (.+)\n?/$1/ if (/^StartChar: /);
		s/FontName: Migu-1M-/FontName: Mu-Iota-Hemiplatia-/;
		s/FullName: Migu 1M /FullName: Mu.Iota Hemiplatia /;
		s/FamilyName: Migu 1M/FamilyName: Mu.Iota Hemiplatia/;
		s/Migu 1M/Mu.Iota Hemiplatia/g if (/LangName: 1041/);
		s/Width: \d+/Width: 500/ if (exists($patchDat{$weight}->{$currentGlyph}));
		if (/^SplineSet/) {
			if (exists($patchDat{$weight}->{$currentGlyph})) {
				print "Patch glyph: $currentGlyph\n";
				print OUT "SplineSet\n";
				print OUT $patchDat{$weight}->{$currentGlyph};
				$skipFlag = 1;
			}
		}
		$skipFlag = 0 if (/^EndSplineSet/);
		print OUT $_ unless ($skipFlag);
	}
	close IN; close OUT;
}

sub sfd2ttf {
	print "Convert font $_[0]...\n";
	my $result = system "fontforge -lang=ff -c \"Open('hemiplatia-$_[0].sfd');Select('.notdef');DetachAndRemoveGlyphs();ScaleToEm(2048);SelectWorthOutputting();RoundToInt();Generate('Mu-Iota-Hemiplatia-$_[0].ttf')\"";
	die "Something went wrong...\n" if ($result);
}

my @subFamilies = ("regular", "bold");
my $srcdir = &extractSourceZip(&determineSourceZip);
&ttf2sfd($srcdir, "migu-1m-$_") foreach (@subFamilies);
&patchGlyph($_) foreach (@subFamilies);
my @subFamilies = ("regular", "bold");
&sfd2ttf($_) foreach (@subFamilies);
print "Done.\n";
